#!/usr/bin/python

import re
import sys
import os
import signal
from enum import Enum
from subprocess import Popen, PIPE
from pyparsing import nestedExpr
from threading import Timer
import pyparsing as pp
import re
import time
import uuid
import logging
from tabulate import tabulate
from texttable import Texttable
import latextable
import csv

rows=[]

def run_all_hwmcc():
    all_filenames = open("test/benchmarks/hwmcc10.txt","r")
    print("opening file, ",all_filenames.name)
    line = all_filenames.readline()
    lineNo=1
    while line:
        filename = line.strip()
        print("reading filename: ", lineNo, line.strip())
        aigcertify("test/hwmcc10/"+filename+".aig")
        line=all_filenames.readline()


def run_all_tip():
    all_filenames = open("test/benchmarks/tip.txt", "r")
    print("opening file, ",all_filenames.name)
    line = all_filenames.readline()
    lineNo=1
    while line:
        filename = line.strip()
        print("reading filename: ", lineNo, line.strip())
        lineNo +=1
        aigcertify("test/tip_paper/"+filename+".aig")
        line=all_filenames.readline()

def run_all_counters(f,k_val):
    filename = "temp/" + f.name.split("/")[-1]
    cmd  = "python3 " +  sys.path[0] + "/../py-aigcertify.py " + " " + filename + " -kis -kwm"

    print("cmd: ", cmd)
    signal.signal(signal.SIGPIPE, signal.SIG_IGN)
    p=Popen(cmd, shell=True, stdout=PIPE)
    output = p.communicate()[0]
    output = output.splitlines()
    result = output[-2].decode('utf8')
    #assert(result == "SUCCESS")
    print(output[-1].decode('utf8'))

def run_all_counters2():
    all_filenames = open("counter/counter_names.txt", "r")
    print("opening file, ",all_filenames.name)
    line = all_filenames.readline()
    lineNo=1
    while line:
        filename = line.strip()
        print("reading filename: ", lineNo, line.strip())
        lineNo +=1
        aigcertify("counter/benchmarks/"+filename+".aag")
        line=all_filenames.readline()

def aigcertify(f):
    global rows 
    cmd  = "python3 " +  sys.path[0] + "/py-aigcertify.py " + " " + f + " -kis "

    print("cmd: ", cmd)
    signal.signal(signal.SIGPIPE, signal.SIG_IGN)
    p=Popen(cmd, shell=True, stdout=PIPE)
    output = p.communicate()[0]
    output = output.splitlines()
    result = output[-2].decode('utf8')
    assert(result == "SUCCESS")

    row = output[-1].decode('utf8')
    print("row:\n",row)
    rows.append(row)

def mcaiger(file, flag_n):
    print("calling Mcaiger...")
    exe = sys.path[0] + "/../mcaiger/mcaiger "
    if(flag_n):
        exe = sys.path[0] + "/../mcaiger/mcaiger " + "-n "

    cmd = exe + "hwmcc10/"+ file+".aig"
    print("cmd: ", cmd)
    
    time1 = time.time()
    signal.signal(signal.SIGPIPE, signal.SIG_IGN)
    p=Popen(cmd, shell=True, stdout=PIPE)
    timer = Timer(90, p.kill)
    try:
        timer.start()
        output = p.communicate()[0]
        time_taken = time.time()-time1
        output = output.splitlines()
        k = output[-1].decode('utf8').split("=")[-1]
    
        print("k: ", k)
        print("McAiger computation time: ", time_taken)
        logger =  logging.getLogger(file)
        logger.info("k="+k)
        logger.info("time="+str(time_taken))

    finally:
        timer.cancel()
        print("McAiger timeout,", file)

    
############ main #############
headers = ["Name", "\#$t$", "\#$C$", "$k$", "\#$C$'","\#$t'$", "\#$c0$", "\#$t0$", "\#$c1$", "\#$t1$", "\#$c2$", "\#$t2$","\#$c3$", "\#$t3$","\#$c4$", "\#$t4$","\#$c5$", "\#$t5$"]
rows.append(headers)


logging.basicConfig(level=logging.DEBUG, format='%(asctime)s %(name)-12s %(message)s', datefmt='%m-%d %H:%M', filename=sys.path[0] + '/2010_aigcertify.txt', filemode='w')
console = logging.StreamHandler()
console.setLevel(logging.INFO)
# set a format which is simpler for console use
formatter = logging.Formatter('%(name)-12s: %(levelname)-8s %(message)s')
# tell the handler to use this format
console.setFormatter(formatter)
# add the handler to the root logger
run_all_hwmcc()
run_all_tip()


